'use strict';

var core = require('@babel/core');
var helperPluginUtils = require('@babel/helper-plugin-utils');

var index = helperPluginUtils.declare((_api, options)=>{
    const replaceOptions = Object.entries(options).sort((a, b)=>b[0].length - a[0].length).map(([searchValue, replaceValue])=>({
            identifier: searchValue.match(/^(typeof\s+)?([A-Za-z_$][\w$]*)/)?.[2] || "",
            searchNode: core.template.expression.ast(searchValue),
            replaceNode: core.template.expression.ast(`${replaceValue}`)
        }));
    return {
        name: "babel-plugin-replace",
        visitor: {
            Expression (path) {
                const { node, scope } = path;
                const replaceOption = replaceOptions.find(({ identifier, searchNode })=>{
                    return core.types.isNodesEquivalent(node, searchNode) && !scope.hasBinding(identifier);
                });
                if (!replaceOption) {
                    return;
                }
                path.replaceWith(core.types.cloneNode(replaceOption.replaceNode));
            }
        }
    };
});

module.exports = index;
